/**
 * Copyright (c) 2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.api;

import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.ow2.petals.jmx.api.api.exception.RouterMonitorServiceErrorException;

/**
 * @author Christophe DENEUX - Linagora
 */
public interface RouterMonitorServiceClient {

    /**
     * Name of the associated MBean
     */
    public static final String ROUTER_MONITOR_MBEAN_NAME = "RouterMonitor";

    /**
     * Clear the monitor storage.The persisted data related to the monitored
     * exchanges are deleted.
     * 
     * @throws RouterMonitorServiceErrorException
     */
    public void clearMonitorStorage() throws RouterMonitorServiceErrorException;

    /**
     * 
     * Get the exchange with the given exchange Id.
     * 
     * @param exchangeId
     * @return The exchange in a key/value form
     * 
     * @throws RouterMonitorServiceErrorException
     */
    public Map<String, String> getExchange(final String exchangeId)
            throws RouterMonitorServiceErrorException;

    /**
     * Get the history of the given exchange.
     * 
     * @param exchangeId
     *            the send/received history
     * @return The history exchange in a key/value form
     * @throws RouterMonitorServiceErrorException
     */
    public Map<String, Long> getExchangeHistory(final String exchangeId)
            throws RouterMonitorServiceErrorException;

    /**
     * Get the list of exchange Ids that has been sent or received at least
     * once, matching the given criteria and the exchange persistence duration
     * 
     * @param interfaceName
     *            The interface name that exchange targets. Can be {@code null}
     * 
     * @param serviceName
     *            The service name that exchange targets. Can be {@code null}
     * 
     * @param endpointName
     *            The endpoint name that exchange targets. Can be {@code null}
     * 
     * @param operationName
     *            The operation name that exchange targets. Can be {@code null}
     * 
     * @param startDate
     *            The start date
     * @param endDate
     *            The end date
     * @return The list of the exchange Ids
     * @throws RouterMonitorServiceErrorException
     */
    public List<String> getExchangeIds(final QName interfaceName, final QName serviceName,
            final String endpointName, final QName operationName, final long startDate,
            final long endDate) throws RouterMonitorServiceErrorException;

    /**
     * Get the list of exchange Ids that has been sent or received at least
     * once, matching the given criteria and the exchange persistence duration
     * 
     * @param propertyKey
     *            The name of the property
     * 
     * @param propertyValue
     *            The value of the property as a {@code String}
     * 
     * @return The list of the exchange Ids
     * @throws RouterMonitorServiceErrorException
     */
    public List<String> getExchangeIds(final String propertyKey, final String propertyValue)
            throws RouterMonitorServiceErrorException;

    /**
     * Get the number of exchange that has been sent or received at least once
     * and matching the duration
     * 
     * @param interfaceName
     *            The interface name that exchange targets. Can be {@code null}
     * 
     * @param serviceName
     *            The service name that exchange targets. Can be {@code null}
     * 
     * @param endpointName
     *            The endpoint name that exchange targets. Can be {@code null}
     * 
     * @param operationName
     *            The operation name that exchange targets. Can be {@code null}
     * 
     * @param startDate
     *            The start date
     * @param endDate
     *            The end date
     * @return The list of the exchange Ids
     * @throws RouterMonitorServiceErrorException
     */
    public int getExchanges(final QName interfaceName, final QName serviceName,
            final String endpointName, final QName operationName, final long startDate,
            final long endDate) throws RouterMonitorServiceErrorException;

    /**
     * Get the list of Router Monitor filter.
     * 
     * @return the list of Router Monitor filter in key/values form.
     * 
     * @throws RouterMonitorServiceErrorException
     */
    public List<Map<String, String>> getMonitoredExchange()
            throws RouterMonitorServiceErrorException;

    /**
     * Get the duration of the monitoring storage. Once the duration is reached,
     * the persisted data related to the monitored exchange are deleted.
     * 
     * @return The duration in milliseconds
     * 
     * @throws RouterMonitorServiceErrorException
     */
    public Long getMonitorStorageDuration() throws RouterMonitorServiceErrorException;

    /**
     * Activate the monitoring of the exchanges matching the given criteria.
     * 
     * @param interfaceName
     *            The interface name that exchange targets. Can be {@code null}
     * 
     * @param serviceName
     *            The service name that exchange targets. Can be {@code null}
     * 
     * @param endpointName
     *            The endpoint name that exchange targets. Can be {@code null}
     * 
     * @param operationName
     *            The operation name that exchange targets. Can be {@code null}
     * 
     * @param storeExchanges
     *            if {@code true}, the exchanges are stored in the Persistence
     *            Service.
     * @throws RouterMonitorServiceErrorException
     */
    public void monitorExchanges(final QName interfaceName, final QName serviceName,
            final String endpointName, final QName operationName, final boolean storeExchanges)
            throws RouterMonitorServiceErrorException;

    /**
     * Set the duration of the monitoring storage. Once the duration is reached,
     * the persisted data related to the monitored exchange are deleted.
     * 
     * @param duration
     *            The duration in millisecond
     * @throws RouterMonitorServiceErrorException
     */
    public void setMonitorStorageDuration(long duration) throws RouterMonitorServiceErrorException;

    /**
     * Deactivate the monitoring of the exchanges matching the given criteria.
     * 
     * @param interfaceName
     *            The interface name that exchange targets. Can be {@code null}
     * 
     * @param ServiceName
     *            The service name that exchange targets. Can be {@code null}
     * 
     * @param endpointName
     *            The endpoint name that exchange targets. Can be {@code null}
     * 
     * @param operationName
     *            The operation name that exchange targets. Can be {@code null}
     * 
     * @throws RouterMonitorServiceErrorException
     */
    public void unmonitorExchanges(QName interfaceName, QName serviceName, String endpointName,
            QName operationName) throws RouterMonitorServiceErrorException;

}
